class TaskPred:

    obj_list = []

    def __init__(self, params):
        self.task_pred_id = params[0].strip()
        self.task_id = int(params[1]) if params[1] else None
        self.pred_task_id = int(params[2]) if params[2] else None
        self.proj_id = params[3].strip()
        self.pred_proj_id = params[4].strip()
        self.pred_type = params[5].strip()
        self.lag_hr_cnt = params[6].strip()
        self.float_path = params[7].strip()
        self.aref = params[8].strip()
        self.arls = params[9].strip()
        TaskPred.obj_list.append(self)

    def get_id(self):
        return self.task_pred_id



    def __repr__(self):
        return str(self.task_id) + '->' + str(self.pred_task_id)


class UDFValue:

    udf_code_id = None
    udf_type_id = None
    fk_id = None
    proj_id = None
    udf_number = None
    udf_text = None

    def __init__(self, params):

        self.udf_type_id = params[0].strip()
        self.fk_id = params[1].strip()
        self.proj_id = params[2].strip()
        self.udf_date = params[3].strip()
        self.udf_number = params[4].strip()
        self.udf_text = params[5].strip()
        self.udf_code_id = params[6].strip()

    def get_id(self):
        return self.udf_type_id

    @staticmethod
    def find_by_id(code_id, activity_code_dict):
        return {k: v for k, v in activity_code_dict.items() if v.actv_code_id == code_id}

    def __repr__(self):
        return self.udf_text + '->' + self.udf_code_id


class ResourceCat:
    obj_list = []

    def __init__(self, params):
        self.rsrc_id = params[0].strip()
        self.rsrc_catg_type_id = params[1].strip()
        self.rsrc_catg_id = params[2].strip()
        ResourceCat.obj_list.append(self)

    def get_id(self):
        return self.rsrc_id

    def __repr__(self):
        return self.rsrc_id + ' has been assign category ' + self.rsrc_catg_id





class Role:
    obj_list = []

    def __init__(self, params):
        self.role_id = int(params[0]) if params[0] else None
        self.parent_role_id = int(params[1]) if params[1] else None
        self.seq_num = int(params[2]) if params[2] else None
        self.role_name = params[3].strip()
        self.role_short_name = params[4].strip()
        self.pobs_id = params[5]
        self.def_cost_qty_link_flag = params[6].strip()
        self.cost_qty_type = params[7].strip()
        self.role_descr = params[8].strip()
        self.last_checksum = params[9].strip()

        Role.obj_list.append(self)

    def __repr__(self):
        return self.role_name



from xerparser.model.tasks import Tasks


class WBS:
    obj_list = []

    def __init__(self, params):
        self.wbs_id = int(params[0].strip()) if params[0] else None
        self.proj_id = int(params[1].strip()) if params[1] else None
        self.obs_id = params[2].strip()
        self.seq_num = params[3].strip()
        self.est_wt = params[4].strip()
        self.proj_node_flag = params[5].strip()
        self.sum_data_flag = params[6].strip()
        self.status_code = params[7].strip()
        self.wbs_short_name = params[8].strip()
        self.wbs_name = params[9].strip()
        self.phase_id = params[10].strip()
        self.parent_wbs_id = int(params[11]) if params[11] else None
        self.ev_user_pct = params[12].strip()
        self.ev_etc_user_value = params[13].strip()
        self.orig_cost = params[14].strip()
        self.indep_remain_total_cost = params[15].strip()
        self.ann_dscnt_rate_pct = params[16].strip()
        self.dscnt_period_type = params[17].strip()
        self.indep_remain_work_qty = params[18].strip()
        self.anticip_start_date = params[19].strip()
        self.anticip_end_date = params[20].strip()
        self.ev_compute_type = params[21].strip()
        self.ev_etc_compute_type = params[22].strip()
        self.guid = params[23].strip()
        self.tmpl_guid = params[24].strip()
        self.plan_open_state = params[25].strip() if len(params) > 25 else None

        WBS.obj_list.append(self)

    def get_id(self):
        return self.wbs_id

    @classmethod
    def get_json(cls):
        root_nodes = list(filter(lambda x: WBS.find_by_id(x.parent_wbs_id) is None, cls.obj_list))
        print(root_nodes)
        json = dict()
        for node in root_nodes:
            json["node"] = node
            json["level"] = 0
            json["childs"] = []
            json["childs"].append(cls.get_childs(node, 0))
        print(json)
        return json

    @classmethod
    def get_childs(cls, node, level):
        nodes_lst = list(filter(lambda x: x.parent_wbs_id == node.wbs_id, cls.obj_list))
        nod = dict()
        for node in nodes_lst:
            nod["node"] = node
            nod["level"] = level + 1
            children = cls.get_childs(node, level + 1)
            nod["childs"] = []
            nod["childs"].append(children)
        return nod
    @classmethod
    def find_by_id(cls, ID):
        obj = list(filter(lambda x: x.wbs_id == ID, cls.obj_list))
        if obj:
            return obj[0]
        return None

    @staticmethod
    def find_by_project_id(project_id, wbs):
        return {k: v for k, v in wbs.items() if v.proj_id == project_id}

    @property
    def activities(self):
        return Tasks.activities_by_wbs_id(self.wbs_id)

    def __repr__(self):
        return self.wbs_name




class SchedOption:
    obj_list = []

    def __init__(self, params):

        self.schedoptions_id = params[0].strip()
        self.proj_id = params[1].strip()
        self.sched_outer_depend_type = params[2].strip()
        self.sched_open_critical_flag = params[3].strip()
        self.sched_lag_early_start_flag = params[4].strip()
        self.sched_retained_logic = params[5].strip()
        self.sched_setplantoforecast = params[6].strip()
        self.sched_float_type = params[7].strip()
        self.sched_calendar_on_relationship_lag = params[8].strip()
        self.sched_use_expect_end_flag = params[9].strip()
        self.sched_progress_override = params[10].strip()
        self.level_float_thrs_cnt = params[11].strip()
        self.level_outer_assign_flag = params[12].strip()
        self.level_outer_assign_priority = params[13].strip()
        self.level_over_alloc_pct = params[14].strip()
        self.level_within_float_flag = params[15].strip()
        self.level_keep_sched_date_flag = params[16].strip()
        self.level_all_rsrc_flag = params[17].strip()
        self.sched_use_project_end_date_for_float = params[18].strip()
        self.enable_multiple_longest_path_calc = params[19].strip()
        self.limit_multiple_longest_path_calc = params[20].strip()
        self.max_multiple_longest_path = params[21].strip()
        self.use_total_float_multiple_longest_paths = params[22].strip()
        self.key_activity_for_multiple_longest_paths = params[23].strip()
        self.LevelPriorityList = params[24].strip()
        SchedOption.obj_list.append(self)

    def get_id(self):
        return self.schedoptions_id

    def __repr__(self):
        return self.proj_id


from xerparser.model.classes.rsrc import Resource

class Resources:

    _rsrcs = []

    def __init__(self):
        self.index = 0

    def add_resource(self, params):
        rsrc = Resource(params)
        self._rsrcs.append(rsrc)

    @staticmethod
    def get_resource_by_id(id):
        rsrc = list(filter(lambda x: x.rsrc_id == id, Resources._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
        else:
            rsrc = None
        return rsrc

    @staticmethod
    def get_parent(id):
        rsrc = list(filter(lambda x: x.rsrc_id == id, Resources._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
            parent = Resources.get_resource_by_id(rsrc.parent_rsrc_id)
        else:
            rsrc = None
        return rsrc


    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= len(self._rsrcs):
            raise StopIteration
        idx = self.index
        self.index +=1
        return self._rsrcs[idx]


class Resource:
    obj_list = []

    def __init__(self, params):
        self.rsrc_id = int(params[0].strip()) if params[0].strip() else None
        self.parent_rsrc_id = int(params[1].strip()) if params[1].strip() else None
        self.clndr_id = int(params[2].strip()) if params[2].strip() else None
        self.role_id = int(params[3].strip()) if params[3].strip() else None
        self.shift_id = params[4].strip()
        self.user_id = params[5].strip()
        self.pobs_id = params[6].strip()
        self.guid = params[7].strip()
        self.rsrc_seq_num = params[8].strip()
        self.email_addr = params[9].strip()
        self.employee_code = params[10].strip()
        self.office_phone = params[11].strip()
        self.other_phone = params[12].strip()
        self.rsrc_name = params[13].strip()
        self.rsrc_short_name = params[14].strip()
        self.rsrc_title_name = params[15].strip()
        self.def_qty_per_hr = params[16].strip()
        self.cost_qty_type = params[17].strip()
        self.ot_factor = params[18].strip()
        self.active_flag = params[19].strip()
        self.auto_compute_act_flag = params[20].strip()
        self.def_cost_qty_link_flag = params[21].strip()
        self.ot_flag = params[22].strip()
        self.curr_id = params[23].strip()
        self.unit_id = params[24].strip()
        self.rsrc_type = params[25].strip()
        self.location_id = params[26].strip()
        self.rsrc_notes = params[27].strip()
        self.load_tasks_flag = params[28].strip()
        self.level_flag = params[29].strip()
        self.last_checksum = params[30].strip()
        Resource.obj_list.append(self)

    def get_id(self):
        return self.rsrc_id

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.rsrc_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    @property
    def parent(self):
        return self.parent_rsrc_id

    def __repr__(self):
        return self.rsrc_name




class UDFType:

    udf_type_id = None
    table_name = None
    udf_type_name = None
    udf_type_label = None
    logical_data_type = None
    super_flag = None
    indicator_expression = None
    summary_indicator_expression = None

    def __init__(self, params):

        self.udf_type_id = params[0].strip()
        self.table_name = params[1].strip()
        self.udf_type_name = params[2].strip()
        self.udf_type_label = params[3].strip()
        self.logical_data_type = params[4].strip()
        self.super_flag = params[5].strip()
        self.indicator_expression = params[6].strip()
        self.summary_indicator_expression = params[7].strip()

    def get_id(self):
        return self.udf_type_id

    def __repr__(self):
        return self.udf_type_name





class ResourceCurve:
    obj_list = []

    def __init__(self, params):
        self.curv_id = int(params[0]) if params[0] else None
        self.curv_name = params[1].strip()
        self.default_flag = params[2]
        self.pct_usage_0 = float(params[3]) if params[3] else None
        self.pct_usage_1 = float(params[4]) if params[4] else None
        self.pct_usage_2 = float(params[5]) if params[5] else None
        self.pct_usage_3 = float(params[6]) if params[6] else None
        self.pct_usage_4 = float(params[7]) if params[7] else None
        self.pct_usage_5 = float(params[8]) if params[8] else None
        self.pct_usage_6 = float(params[9]) if params[9] else None
        self.pct_usage_7 = float(params[10]) if params[10] else None
        self.pct_usage_8 = float(params[11]) if params[11] else None
        self.pct_usage_9 = float(params[12]) if params[12] else None
        self.pct_usage_10 = float(params[13]) if params[13] else None
        self.pct_usage_11 = float(params[14]) if params[14] else None
        self.pct_usage_12 = float(params[15]) if params[15] else None
        self.pct_usage_13 = float(params[16]) if params[16] else None
        self.pct_usage_14 = float(params[17]) if params[17] else None
        self.pct_usage_15 = float(params[18]) if params[18] else None
        self.pct_usage_16 = float(params[19]) if params[19] else None
        self.pct_usage_17 = float(params[20]) if params[20] else None
        self.pct_usage_18 = float(params[21]) if params[21] else None
        self.pct_usage_19 = float(params[22]) if params[22] else None
        self.pct_usage_20 = float(params[23]) if params[23] else None

        ResourceCurve.obj_list.append(self)

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.curv_id == id, cls.obj_list))[0]
        return obj

    def __repr__(self):
        return self.curv_name



from xerparser.model.wbss import WBSs
from xerparser.model.tasks import Tasks

class Project:
    obj_list = []

    def __init__(self, params):

        self.proj_id = int(params[0].strip()) if params[0].strip() else None
        self.fy_start_month_num = params[1].strip()
        self.rsrc_self_add_flag = params[2].strip()
        self.allow_complete_flag = params[3].strip()
        self.rsrc_multi_assign_flag = params[4].strip()
        self.checkout_flag = params[5].strip()
        self.project_flag = params[6].strip()
        self.step_complete_flag = params[7].strip()
        self.cost_qty_recalc_flag = params[8].strip()
        self.batch_sum_flag = params[9].strip()
        self.name_sep_char = params[10].strip()
        self.def_complete_pct_type = params[11].strip()
        self.proj_short_name = params[12].strip()
        self.acct_id = params[13].strip()
        self.orig_proj_id = params[14].strip()
        self.source_proj_id = params[15].strip()
        self.base_type_id = params[16].strip()
        self.clndr_id = params[17].strip()
        self.sum_base_proj_id = params[18].strip()
        self.task_code_base = params[19].strip()
        self.task_code_step = params[20].strip()
        self.priority_num = params[21].strip()
        self.wbs_max_sum_level = params[22].strip()
        self.strgy_priority_num = params[23].strip()
        self.last_checksum = params[24].strip()
        self.critical_drtn_hr_cnt = params[25].strip()
        self.def_cost_per_qty = params[26].strip()
        self.last_recalc_date = params[27].strip()
        self.plan_start_date = params[28].strip()
        self.plan_end_date = params[29].strip()
        self.scd_end_date = params[30].strip()
        self.add_date = params[31].strip()
        self.last_tasksum_date = params[32].strip()
        self.fcst_start_date = params[33].strip()
        self.def_duration_type = params[34].strip()
        self.task_code_prefix = params[35].strip()
        self.guid = params[36].strip()
        self.def_qty_type = params[37].strip()
        self.add_by_name = params[38].strip()
        self.web_local_root_path = params[39].strip()
        self.proj_url = params[40].strip()
        self.def_rate_type = params[41].strip()
        self.add_act_remain_flag = params[42].strip()
        self.act_this_per_link_flag = params[43].strip()
        self.def_task_type = params[44].strip()
        self.act_pct_link_flag = params[45].strip()
        self.critical_path_type = params[46].strip()
        self.task_code_prefix_flag = params[47].strip()
        self.def_rollup_dates_flag = params[48].strip()
        self.use_project_baseline_flag = params[49].strip()
        self.rem_target_link_flag = params[50].strip()
        self.reset_planned_flag = params[51].strip()
        self.allow_neg_act_flag = params[52].strip()
        self.sum_assign_level = params[53].strip()
        self.last_fin_dates_id = params[54].strip()
        self.last_baseline_update_date = params[55].strip()
        self.cr_external_key = params[56].strip()
        self.apply_actuals_date = params[57].strip()
        self.location_id = params[58].strip()
        self.loaded_scope_level = params[59].strip()
        self.export_flag = params[60].strip()
        self.new_fin_dates_id = params[61].strip()
        self.baselines_to_export = params[62].strip()
        self.baseline_names_to_export = params[63].strip()
        self.next_data_date = params[64].strip()
        self.close_period_flag = params[65].strip()
        self.sum_refresh_date = params[66].strip() if len(params)> 66 else None
        self.trsrcsum_loaded = params[67].strip() if len(params) > 67 else None
        Project.obj_list.append(self)

    @property
    def id(self):
        return self.proj_id


    @property
    def activities(self):
        return Tasks.get_by_project(self.proj_id)

    @property
    def wbss(self):
        return WBSs.get_by_project(self.proj_id)

    @property
    def resources(self):
        return Resources.get_by_project(self.proj_id
                                        )
    def __repr__(self):
        return self.proj_short_name


class Currency:
    obj_list = []

    def __init__(self, params):
        # Unique ID generated by the system.
        self.curr_id = int(params[0]) if params[0] else None
        # The number of decimal places displayed.
        self.decimal_digit_cnt = int(params[1]) if params[1] else None
        # The symbol used to identify each defined currency.
        self.curr_symbol = params[2].strip()
        # The decimal symbol displayed.
        self.decimal_symbol = params[3].strip()
        # The symbol used to group the numbers.
        self.digit_group_symbol = params[4].strip()
        # The symbol used to display a positive currency.
        self.pos_curr_fmt_type = params[5].strip()
        # The symbol used to display a negative currency.
        self.neg_curr_fmt_type = params[6].strip()
        # The names of all defined currencies.
        self.curr_type = params[7].strip()
        # The identifiers for all currencies defined in Project Management.
        self.curr_short_name = params[8].strip()
        # Currency Group Digit Count
        self.group_digit_cnt = params[9].strip()
        # The current exchange rate between the selected currency and the base currency.
        self.base_exch_rate = params[10].strip()

        Currency.obj_list.append(self)

    def get_id(self):
        return self.curr_id


    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.curr_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return self.curr_symbol + ' ' + self.curr_type


class TaskRsrc:
    taskrsrc_id = None
    task_id = None
    proj_id = None
    cost_qty_link_flag = None
    role_id = None
    acct_id = None
    rsrc_id = None
    pobs_id = None
    skill_level = None
    remain_qty = None
    target_qty = None
    remain_qty_per_hr = None
    target_lag_drtn_hr_cnt = None
    target_qty_per_hr = None
    act_ot_qty = None
    act_reg_qty = None
    relag_drtn_hr_cnt = None
    ot_factor = None
    cost_per_qty = None
    target_cost = None
    act_reg_cost = None
    act_ot_cost = None
    remain_cost = None
    act_start_date = None
    act_end_date = None
    restart_date = None
    reend_date = None
    target_start_date = None
    target_end_date = None
    rem_late_start_date = None
    rem_late_end_date = None
    rollup_dates_flag = None
    target_crv = None
    remain_crv = None
    actual_crv = None
    ts_pend_act_end_flag = None
    guid = None
    rate_type = None
    act_this_per_cost = None
    act_this_per_qty = None
    curv_id = None
    rsrc_type = None
    cost_per_qty_source_type = None
    create_user = None
    create_date = None
    has_rsrchours = None
    taskrsrc_sum_id = None

    def __init__(self, params):
        self.taskrsrc_id = params[0].strip()
        self.task_id = params[1].strip()
        self.proj_id = params[2].strip()
        self.cost_qty_link_flag = params[3].strip()
        self.role_id = params[4].strip()
        self.acct_id = params[5].strip()
        self.rsrc_id = params[6].strip()
        self.pobs_id = params[7].strip()
        self.skill_level = params[8].strip()
        self.remain_qty = params[9].strip()
        self.target_qty = params[10].strip()
        self.remain_qty_per_hr = params[11].strip()
        self.target_lag_drtn_hr_cnt = params[12].strip()
        self.target_qty_per_hr = params[13].strip()
        self.act_ot_qty = params[14].strip()
        self.act_reg_qty = params[15].strip()
        self.relag_drtn_hr_cnt = params[16].strip()
        self.ot_factor = params[17].strip()
        self.cost_per_qty = params[18].strip()
        self.target_cost = params[19].strip()
        self.act_reg_cost = params[20].strip()
        self.act_ot_cost = params[21].strip()
        self.remain_cost = params[22].strip()
        self.act_start_date = params[23].strip()
        self.act_end_date = params[24].strip()
        self.restart_date = params[25].strip()
        self.reend_date = params[26].strip()
        self.target_start_date = params[27].strip()
        self.target_end_date = params[28].strip()
        self.rem_late_start_date = params[29].strip()
        self.rem_late_end_date = params[30].strip()
        self.rollup_dates_flag = params[31].strip()
        self.target_crv = params[32].strip()
        self.remain_crv = params[33].strip()
        self.actual_crv = params[34].strip()
        self.ts_pend_act_end_flag = params[35].strip()
        self.guid = params[36].strip()
        self.rate_type = params[37].strip()
        self.act_this_per_cost = params[38].strip()
        self.act_this_per_qty = params[39].strip()
        self.curv_id = params[40].strip()
        self.rsrc_type = params[41].strip()
        self.cost_per_qty_source_type = params[42].strip()
        self.create_user = params[43].strip()
        self.create_date = params[44].strip()
        self.has_rsrchours = params[45].strip()
        self.taskrsrc_sum_id = params[46].strip()

    def get_id(self):
        return self.taskrsrc_id

    @staticmethod
    def find_by_id(code_id, activity_code_dict):
        return {k: v for k, v in activity_code_dict.items() if v.actv_code_id == code_id}

    def __repr__(self):
        return self.task_id + '->' + self.rsrc_id + ' = ' + self.target_qty
